from os import environ as env
import requests
from time import sleep


sleep_seconds = float(env['SLEEP_SECONDS']) if 'SLEEP_SECONDS' in env else 15  # noqa: 501


def prometheus_get(kwargs):
    api_path = "/prometheus/get"
    url = f"{kwargs['frigga_url']}{api_path}"
    resp = requests.get(
        url=url,
        params={
            "prom_url": kwargs['prom_url'],
            "raw": True
        }
    )
    if resp.status_code == 200:
        data = int(resp.text)
        return data
    else:
        return resp.text


def grafana_list(kwargs):
    api_path = "/grafana/list"
    url = f"{kwargs['frigga_url']}{api_path}"
    resp = requests.post(
        url=url,
        data={
            "base_url": kwargs['grafana_url'],
            "api_key": kwargs['grafana_api_key'],
            "output_file_path": kwargs['metrics_json_path']
        },
        timeout=15
    )
    data = resp.text
    return data


def prometheus_apply(kwargs):
    api_path = "/prometheus/apply"
    url = f"{kwargs['frigga_url']}{api_path}"
    resp = requests.post(
        url=url,
        data={
            "prom_yaml_path": kwargs['prom_yaml_path'],
            "metrics_json_path": kwargs['metrics_json_path'],
            "create_backup_file": kwargs['create_backup_file'],
            "skip_rules_file": kwargs['skip_rules_file'],
        }
    )
    data = resp.text
    return data


def prometheus_reload(kwargs):
    api_path = "/prometheus/reload"
    url = f"{kwargs['frigga_url']}{api_path}"
    resp = requests.post(url=url, data={
        "prom_url": kwargs['prom_url'],
        "raw": kwargs['raw'],
    })
    data = resp.text
    return data


def main(kwargs):
    num_dataseries_before = prometheus_get(kwargs)
    print(num_dataseries_before)
    metrics_json = grafana_list(kwargs)
    print(metrics_json)
    prom_apply_results = prometheus_apply(kwargs)
    print(prom_apply_results)
    prom_reload_results = prometheus_reload(kwargs)
    print(prom_reload_results)
    print(f"Sleeping for {sleep_seconds} seconds ...")
    sleep(sleep_seconds)
    num_dataseries_after = prometheus_get(kwargs)
    print(num_dataseries_after)
    if num_dataseries_after < num_dataseries_before:
        decreased_precentage = round(
            100 * (1 - (num_dataseries_after/num_dataseries_before)), 2)
        print(
            f"decreased from {num_dataseries_before} to {num_dataseries_after}, {decreased_precentage}%"  # noqa: 501
        )
    elif num_dataseries_after == num_dataseries_before:
        print(f"no change - {num_dataseries_after}")
    else:
        raise Exception("something went wrong")


if __name__ == '__main__':
    main()



import asyncio
import websockets
import json
import logging

from .prometheus import get_total_dataseries as prometheus_get
from .grafana import get_metrics_list as grafana_list
from .prometheus import apply_yaml as prometheus_apply
from .prometheus import reload_prom as prometheus_reload

logging.basicConfig()
logger = logging.getLogger('websockets')
logger.setLevel(logging.INFO)


async def msg(websocket, path):
    received = await websocket.recv()
    try:
        received = json.loads(received)
    except Exception:
        pass

    print(f"Requested: {received}")
    message = ""
    if 'action' in received:
        try:
            if received['action'] == "prometheus_get":
                received.pop('action')
                message = prometheus_get(**received)
                message = f">> [LOG] Current number of metrics - {message}"
            elif received['action'] == "grafana_list":
                received.pop('action')
                message = grafana_list(**received)
                message = f">> [LOG] Successfully created {received['output_file_path']}"  # noqa: 501
            elif received['action'] == "prometheus_apply":
                received.pop('action')
                message = prometheus_apply(**received)
                message = ">> [LOG] Successfully applied new relabel rules in Prometheus. Reload Prometheus to apply the changes."  # noqa: 501
            elif received['action'] == "prometheus_reload":
                received.pop('action')
                message = prometheus_reload(**received)
                message = ">> [LOG] Successfully reloaded Prometheus"
        except Exception as error:
            logger.error(error)
            await websocket.close(code=1011, reason=error.__str__()[0:120])

    else:
        message = f"Hello {received}!"
    message = json.dumps(message)
    await websocket.send(message)
    print(f"Sent back: {message}")


def run_wss(port=8085, debug=False):
    start_server = websockets.serve(msg, "0.0.0.0", port)
    asyncio.get_event_loop().run_until_complete(start_server)
    asyncio.get_event_loop().run_forever()


if __name__ == '__main__':
    run_wss()



import json
import requests
from time import time

from bs4 import BeautifulSoup
import yaml

from .config import print_msg


def request_words(url, selector, replace_str="()", replace_with=""):
    response = requests.get(url)
    if 200 <= response.status_code < 400:
        print_msg(
            msg_content=f"Successfully got words from {url}", msg_type='log')
    else:
        print_msg(
            msg_content=f"Failed to reach {url}, have you provided the '--web.enable-admin-api' in Prometheus?"  # noqa: 501
        )
    html_page_text = response.text
    soup = BeautifulSoup(html_page_text, 'html.parser')
    return [
        item.string.replace(replace_str, replace_with)
        for item in soup.select(selector)
    ]


def get_ignored_words():
    """Get the list of words to ignore when scraping from Grafana"""
    print_msg(
        msg_content="Getting the list of words to ignore when scraping from Grafana"  # noqa: 501
    )
    words_list = []

    prometheus_urls = [
        {
            "url": "https://prometheus.io/docs/prometheus/latest/querying/functions/",  # noqa: 501
            "selectors": ".toc-right ul li code"
        },
        {
            "url": "https://prometheus.io/docs/prometheus/latest/querying/operators/",  # noqa: 501
            "selectors": "ul li code"
        }
    ]
    for item in prometheus_urls:
        words_list += request_words(item["url"], item["selectors"])
    return sorted(list(set(words_list)))


def apply_yaml(prom_yaml_path, metrics_json_path, create_backup_file=True, skip_rules_file=False):  # noqa: 501
    print_msg(msg_content=f"Reading documents from {prom_yaml_path}")
    with open(prom_yaml_path, "r") as fp:
        prom_yaml = fp.read()
    prom_documents = yaml.safe_load_all(prom_yaml)
    prom_doc = None
    frigga_doc = None
    for doc in prom_documents:
        if 'scrape_configs' in doc:
            prom_doc = doc
        elif 'name' in doc and doc['name'] == 'frigga':
            frigga_doc = doc
    if not prom_doc:
        print_msg(
            msg_content="Missing 'scrape_configs' in prometheus.yml",
            msg_type="error"
        )
    if not frigga_doc:
        print_msg(
            msg_content="Missing frigga document in prometheus.yml",
            msg_type="error"
        )
    print_msg(msg_content="Found relevant documents")
    print_msg(msg_content="Loading metrics.json")
    with open(metrics_json_path, "r") as file:
        metrics_dict = json.load(file)

    print_msg(msg_content="Generating metrics_relabel_configs in memory")
    relabel_configs = []
    for metric in metrics_dict['all_metrics']:
        relabel_configs.append({
            "source_labels": ['__name__'],
            "regex": f"^{metric}",
            "target_label": "__tmp_keep_me",
            "replacement": True
        })
    relabel_configs.append({
        "source_labels": ["__tmp_keep_me"],
        "regex": True,
        "action": "keep"
    })

    noalias_dumper = yaml.dumper.SafeDumper
    noalias_dumper.ignore_aliases = lambda self, data: True

    # write relabel configs to file
    if not skip_rules_file:
        rules_file_path = ".prometheus-rules.yml"
        print_msg(
            msg_content=f"Writing relabel_configs to {rules_file_path}")
        with open(rules_file_path, 'w') as fs:
            yaml.dump_all(
                documents=[relabel_configs],
                stream=fs, default_flow_style=False,
                Dumper=noalias_dumper,
                indent=2
            )

    # add relabel configs to all jobs, except the ones in exclude_jobs
    for job in prom_doc['scrape_configs']:
        if job['job_name'] not in frigga_doc['exclude_jobs']:
            job['metric_relabel_configs'] = relabel_configs

    # backup old prom yaml
    if create_backup_file:
        print_msg(msg_content=f"Creating a backup file for {prom_yaml_path}")
        with open(prom_yaml_path, 'r') as source_fs:
            with open("prometheus.yml.bak.yml", 'w') as target_fs:
                target_fs.write(source_fs.read())

    # write new prom frigga yaml
    print_msg(
        msg_content=f"Writing the new metrics_relabel_configs to {prom_yaml_path}"  # noqa: 501
    )
    with open(prom_yaml_path, 'w') as fs:
        yaml.dump_all(
            documents=[prom_doc, frigga_doc],
            stream=fs, default_flow_style=False,
            Dumper=noalias_dumper,
            indent=2
        )
    msg = f"Done! Now reload {prom_yaml_path} with 'frigga pr -u http://localhost:9090'"  # noqa: 501
    print_msg(msg_content=msg)
    return msg


def reload_prom(prom_url="http://localhost:9090", raw=False):
    api_path = "/-/reload"
    url = f"{prom_url}{api_path}"
    response = requests.post(url, allow_redirects=True)
    if 200 <= response.status_code < 204:
        data = response.status_code
        if raw:
            print(data)
            return data
        else:
            print_msg(
                msg_content=f"Successfully reloaded Prometheus - {url}",
                msg_type='log'
            )
            return {
                "status": data,
                "prom_url": url
            }
    else:
        print_msg(
            msg_content=f"Failed to reload Prometheus - {url}, have you provided the '--web.enable-admin-api' in Prometheus?",  # noqa:501
            msg_type='error',
            data=response.text
        )
    return True


def get_total_dataseries(prom_url="http://localhost:9090", raw=False):
    api_path = "/api/v1/query"
    target_url = f"{prom_url}{api_path}"
    query = "sum(scrape_samples_post_metric_relabeling)"
    timestamp = int(time())
    query_string_parameters = f"query={query}&start={timestamp}"
    url = f"{target_url}?{query_string_parameters}"

    try:
        response = requests.get(url, allow_redirects=True)
    except Exception as error:
        print_msg(
            msg_content=error.__str__(),
            msg_type='error'
        )
    if 200 <= response.status_code < 204:
        try:
            resp = response.json()
            data = int(resp['data']['result'][0]['value'][1])
        except Exception as e:
            print_msg(
                msg_content=f"Unknown response\n{e}",
                data=resp,
                msg_type="error"
            )
        if raw:
            print(data)
            return data
        else:
            print_msg(
                msg_content=f"Total number of data-series: {data}",
                msg_type='log'
            )
            return {"num_data_series": data}
    else:
        print_msg(
            msg_content=f"Failed to get total number of data-series. Is Prometheus reachable? - {url}",  # noqa:501
            msg_type='error',
            data=response.text
        )
    return True



from os import getcwd, path, environ
from setuptools import setup
import subprocess


# Change accordingly ----------------------------
PACKAGE_DIR_NAME = "frigga"
GITHUB_OWNER = "unfor19"
GITHUB_REPOSITORY = "frigga"
# -----------------------------------------------


# Keep the same structure, should NOT be changed
def get_short_commit():
    """Assumes git is installed"""
    return subprocess.check_output(
        ['git', 'rev-parse', '--short', 'HEAD']
    ).decode().strip()


# Keep the same structure, should NOT be changed
with open("version", "r") as fh:
    version = fh.read()

with open(path.join(getcwd(), 'src', PACKAGE_DIR_NAME, '__init__.py'), "w") as fh:  # noqa: E501
    short_commit = ""
    try:
        short_commit = get_short_commit()
    except Exception:
        try:
            if "GITHUB_SHA" in environ and environ["GITHUB_SHA"]:
                short_commit = environ["GITHUB_SHA"]
        except Exception:
            short_commit = "abcd123"
    finally:
        short_commit = short_commit[0:7]

    fh.write(
        f"__version__ = '{version}'\n__short_commit__ = '{short_commit}'"
    )

setup(
    version=version,
    download_url=f'https://github.com/{GITHUB_OWNER}/{GITHUB_REPOSITORY}/archive/{version}.tar.gz',  # noqa: E501
)
# -----------------------------------------------



import logging

from flask import Flask, request, jsonify, abort, render_template
from waitress import serve

from .prometheus import reload_prom as prometheus_reload
from .prometheus import apply_yaml as prometheus_apply
from .prometheus import get_total_dataseries as prometheus_get
from .grafana import get_metrics_list as grafana_list


app = Flask(__name__)
logging.basicConfig()
logger = logging.getLogger('waitress')
logger.setLevel(logging.INFO)


@app.errorhandler(500)
def internal_server_error(e):
    logger.error(e)
    return jsonify(error=str(e)), 500


@app.route('/', methods=['GET', ])
def root():
    logger.info(request)
    try:
        rendered_template = render_template('index.html')
        return rendered_template
    except Exception as e:
        abort(500, e)


@app.route('/grafana/list', methods=['POST', ])
def list_grafana_metrics():
    logger.info(request)
    args = {
        "base_url": request.form['base_url'],
        "api_key": request.form['api_key'],
        "output_file_path": request.form['output_file_path']
    }
    try:
        grafana_list(**args)
    except Exception as e:
        abort(500, e)
    return f"Created {args['output_file_path']}"


@app.route('/prometheus/get', methods=['GET'])
def get_prometheus_dataseries_num():
    logger.info(request)
    args = {
        "prom_url": request.args.get('prom_url'),
        "raw": request.args.get('raw')
    }
    try:
        data = prometheus_get(**args)
        logger.info(data)
    except Exception as e:
        abort(500, e)

    return str(data)


@app.route('/prometheus/apply', methods=['POST', ])
def apply_prometheus():
    logger.info(request)
    args = {
        "prom_yaml_path": request.form['prom_yaml_path'],
        "metrics_json_path": request.form['metrics_json_path'],
        "create_backup_file": request.form['create_backup_file'],
        "skip_rules_file": request.form['skip_rules_file']
    }
    try:
        data = prometheus_apply(**args)
    except Exception as e:
        abort(500, e)
    print(data)
    return data


@app.route('/prometheus/reload', methods=['POST', ])
def reload_prometheus():
    logger.info(request)
    args = {
        "prom_url": request.form['prom_url'],
        "raw": request.form['raw']
    }
    try:
        data = prometheus_reload(**args)
    except Exception as e:
        abort(500, e)
    print(data)
    return "reloaded"


def shutdown_server():
    func = request.environ.get('werkzeug.server.shutdown')
    if func is None:
        raise RuntimeError('Not running with the Werkzeug Server')
    func()


@app.route('/stop', methods=['POST', ])
def webserver_stop():
    logger.info(request)
    shutdown_server()
    return 'frigga webserver is shutting down...'


def run(port=8083, debug=False):
    app.config['TESTING'] = debug
    logger.info(app)
    if debug:
        app.run(host="0.0.0.0", port=port)
    else:
        serve(app, host="0.0.0.0", port=port)


if __name__ == '__main__':
    run()



import click


from .grafana import get_metrics_list
from .config import print_msg, is_docker, pass_config
from .prometheus import apply_yaml, reload_prom, get_total_dataseries
from .webserver import run as run_webserver
from .client import main as run_client
from .wsserver import run_wss as run_websocket_webserver
from .wsclient import main as run_websocket_client


class AliasedGroup(click.Group):
    def get_command(self, ctx, cmd_name):
        app_aliases = {
            "c": "client",
            "m": "main",
            "p": "prometheus",
            "g": "grafana",
            "w": "webserver"
        }
        action_aliases = {
            "a": "apply",
            "g": "get",
            "d": "delete",
            "l": "list",
            "r": "reload",
            "s": "start"
        }
        if len(cmd_name) == 2:
            words = []
            if cmd_name[0] in app_aliases:
                words.append(app_aliases[cmd_name[0]])
            if cmd_name[1] in action_aliases:
                words.append(action_aliases[cmd_name[1]])
            if len(words) == 2:
                cmd_name = "-".join(words)

        rv = click.Group.get_command(self, ctx, cmd_name)
        if rv is not None:
            return rv
        matches = [x for x in self.list_commands(ctx)
                   if x.startswith(cmd_name)]
        if not matches:
            return None
        elif len(matches) == 1:
            return click.Group.get_command(self, ctx, matches[0])
        ctx.fail(f"Too many matches: {', '.join(sorted(matches))}")


@click.command(cls=AliasedGroup)
@pass_config
@click.option(
    '--ci', '-ci',
    is_flag=True, help="Use this flag to avoid confirmation prompts"
)
def cli(config, ci):
    """No confirmation prompts"""
    if is_docker():
        ci = True
    config.ci = ci


@cli.command()
@click.option(
    '--grafana-url', '-gurl',
    default='http://localhost:3000',
    prompt=True, required=False, show_default=True, type=str
)
@click.option(
    '--grafana-api-key', '-gkey',
    prompt=True, required=True, hide_input=True, type=str)
@click.option(
    '--output-file-path', '-o',
    default='./.metrics.json',
    show_default=True, required=False, type=str
)
def grafana_list(grafana_url, grafana_api_key, output_file_path):
    """Alias: gl\n
Provide Grafana URL and Grafana API Key (Viewer)\n
Returns a list of metrics that are used in all dashboards"""
    if "http" not in grafana_url:
        print_msg(
            msg_content="Grafana URL must contain 'http' or 'https'",
            msg_type="error"
        )
    get_metrics_list(grafana_url, grafana_api_key, output_file_path)


@cli.command()
@click.option(
    '--prom-yaml-path',
    '-ppath',
    default='docker-compose/prometheus.yml',
    prompt=True, required=True, show_default=False, type=str
)
@click.option(
    '--metrics-json-path', '-mjpath',
    default='./.metrics.json',
    show_default=True, prompt=True, required=False, type=str
)
@click.option(
    '--create-backup-file', '-b',
    is_flag=True, default=True, required=False
)
@click.option(
    '--skip-rules-file', '-sr',
    is_flag=True, default=False, required=False
)
def prometheus_apply(prom_yaml_path, metrics_json_path, create_backup_file, skip_rules_file):  # noqa: 501
    """Alias: pa\n
Applies .metrics.json for a given prometheus.yml file\n

By default:\n
- Creates a backup of prometheus.yml to prometheus.yml.bak.yml (same dir as prometheus.yml)\n
- Creates a .prometheus-rules.yml file with all relabel_configs
    """  # noqa: 501
    apply_yaml(prom_yaml_path, metrics_json_path,
               create_backup_file, skip_rules_file)


@cli.command()
@click.option(
    '--prom-url', '-u',
    default='http://localhost:9090',
    prompt=True, required=True, show_default=False, type=str
)
@click.option(
    '--raw', '-r',
    is_flag=True, default=False, required=False
)
def prometheus_reload(prom_url, raw):
    """Alias: pr\n
    Reload Prometheus
    """
    reload_prom(prom_url, raw)


@cli.command()
@click.option(
    '--prom-url', '-u',
    default='http://localhost:9090',
    prompt=True, required=True, show_default=False, type=str
)
@click.option(
    '--raw', '-r',
    is_flag=True, default=False, required=False
)
def prometheus_get(prom_url, raw):
    """Alias: pg\n
    Get total number of data series
    """
    get_total_dataseries(prom_url, raw)


@cli.command()
@click.option(
    '--debug', '-d',
    is_flag=True,
    prompt=False, required=False, type=int
)
@click.option(
    '--port', '-p',
    default=8085,
    prompt=False, required=False, type=int
)
@click.option(
    '--use-http',
    is_flag=True,
    default=False,
    prompt=False, required=False
)
def webserver_start(debug, port, use_http):
    """Alias: ws\n
Runs a webserver that will execute frigga's commands, according to the client's requests\n
The webserver should have network access to Grafana and Prometheus instances, and read/write permission to the prometheus.yml file.\n
By default:\n
- Runs as a WebSockets server, to use an HTTP server, add the `--use-http` flag
    """  # noqa: 501
    if not use_http:
        run_websocket_webserver(port, debug)
    else:
        run_webserver(port, debug)


@cli.command()
@click.option(
    '--prom-url', '-u',
    default='http://localhost:9090',
    required=False, type=str
)
@click.option(
    '--raw', '-r',
    is_flag=True, default=False, required=False
)
@click.option(
    '--metrics-json-path', '-mjpath',
    default='./.metrics.json',
    required=False, type=str
)
@click.option(
    '--create-backup-file', '-b',
    is_flag=True, default=True, required=False
)
@click.option(
    '--skip-rules-file', '-sr',
    is_flag=True, default=False, required=False
)
@click.option(
    '--grafana-url', '-gurl',
    default='http://localhost:3000',
    required=False, type=str
)
@click.option(
    '--frigga-url', '-gurl',
    default='ws://localhost:8085',
    required=False, type=str
)
@click.option(
    '--grafana-api-key', '-gkey',
    required=False, type=str)
@click.option(
    '--output-file-path', '-o',
    default='./.metrics.json',
    required=False, type=str
)
@click.option(
    '--prom-yaml-path',
    '-ppath',
    default='prometheus.yml',
    required=False, type=str
)
@click.option(
    '--use-http',
    is_flag=True,
    default=False,
    prompt=False, required=False
)
def client_start(use_http, **kwargs):
    """Alias: cs\n
    Runs a WebSockets client, to use HTTP, add `--use-http`\n
    - Order of commands- prometheus-get (before change), grafana-list, prometheus-apply, prometheus-get (after change)\n
    - Priority of args:\n
    client arguments > client env vars > server arguments > server env vars"""  # noqa: 501

    if not use_http:
        run_websocket_client(kwargs)
    else:
        run_client(kwargs)


@cli.command()
def version():
    """Print the installed version"""
    from .__init__ import __version__ as version
    from .__init__ import __short_commit__ as short_commit

    if version == "99.99.99":
        version = "docker"
    elif version == "0.0.1":
        version = "local"

    if short_commit == "abcd123":
        short_commit = "unknown_commit_hash"

    msg = f"{version} {short_commit}"
    print(msg)



from os import environ as env

import asyncio
import websockets
import json
from time import sleep

from .config import print_msg

sleep_seconds = float(env['SLEEP_SECONDS']) if 'SLEEP_SECONDS' in env else 30  # noqa: 501


async def start_websocket(kwargs):
    num_dataseries_before = await send_message({
        "action": "prometheus_get",
        "prom_url": kwargs['prom_url'],
        "raw": kwargs['raw'],
        "frigga_url": kwargs['frigga_url'],
    })

    await send_message({
        "action": "grafana_list",
        "base_url": kwargs['grafana_url'],
        "api_key": kwargs['grafana_api_key'],
        "output_file_path": kwargs['metrics_json_path'],
        "frigga_url": kwargs['frigga_url'],
    })

    await send_message({
        "action": "prometheus_apply",
        "prom_yaml_path": kwargs['prom_yaml_path'],
        "metrics_json_path": kwargs['metrics_json_path'],
        "create_backup_file": kwargs['create_backup_file'],
        "skip_rules_file": kwargs['skip_rules_file'],
        "frigga_url": kwargs['frigga_url'],
    })

    await send_message({
        "action": "prometheus_reload",
        "prom_url": kwargs['prom_url'],
        "raw": True,
        "frigga_url": kwargs['frigga_url'],
    })
    print_msg(msg_content=f"Sleeping {sleep_seconds} seconds ...")
    sleep(sleep_seconds)

    num_dataseries_after = await send_message({
        "action": "prometheus_get",
        "prom_url": kwargs['prom_url'],
        "raw": True,
        "frigga_url": kwargs['frigga_url']
    })

    num_dataseries_before = int(
        num_dataseries_before.__str__().split("-")[1].strip())
    num_dataseries_after = int(
        num_dataseries_after.__str__().split("-")[1].strip())

    if num_dataseries_after < num_dataseries_before:
        decreased_precentage = round(
            100 * (1 - (num_dataseries_after/num_dataseries_before)), 2)
        print_msg(
            f"Successfully decreased from {num_dataseries_before} to {num_dataseries_after}, {decreased_precentage}%"  # noqa: 501
        )
    elif num_dataseries_after == num_dataseries_before:
        print_msg(f"No change - {num_dataseries_after}")
    else:
        print(f">> [ERROR] Before ({num_dataseries_before}) is smaller than after ({num_dataseries_after})")  # noqa: 501
        exit(1)


async def send_message(kwargs):
    uri = kwargs['frigga_url']
    if 'frigga_url' in kwargs:
        kwargs.pop('frigga_url')
    async with websockets.connect(uri) as websocket:

        await websocket.send(json.dumps(kwargs))
        response = None
        try:
            response = await websocket.recv()
        except Exception as error:
            print(error)
            exit()

        try:
            response = json.loads(response)
        except Exception:
            pass

        print(f"{response}")
        return response


def main(kwargs):
    asyncio.get_event_loop().run_until_complete(asyncio.gather((start_websocket(kwargs))))  # noqa: 501


if __name__ == '__main__':
    main()
